#include "stdafx.h"
#include "LeoHelpers_String.h"
#include "LeoHelpers_Scope.h"
#include "LeoHelpers_Misc.h"
#include "LeoHelpers_Opus.h"
#include "LeoHelpers_Window.h"
#include "LeoHelpers_Layout.h"
#include "LeoHelpers_Dialog.h"

#ifdef _DEBUG
//#define LEOHELPERS_DEBUG_TABBACKGROUND
#endif


BOOL LeoHelpers::GetDlgItemRect(HWND hwndDlg, int nIDDlgItem, RECT *pRect)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		bResult = ::GetWindowRect(hwnd, pRect);
	}

	return(bResult);
}

BOOL LeoHelpers::OffsetDlgItem(HWND hwndDlg, int nIDDlgItem, LONG dx, LONG dy, LONG dw, LONG dh, BOOL bRepaint)
{
	BOOL bResult = FALSE;
	HWND hwnd = hwndDlg; // We'll adjust the dialog itself if nIDDlgItem is zero.

	if (0 != nIDDlgItem && NULL != hwndDlg)
	{
		hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);
	}

	if (NULL != hwnd)
	{
		RECT r;
		GetWindowRect(hwnd, &r);

		if (0 == nIDDlgItem || ScreenToClientRect(hwndDlg, &r))
		{
			bResult = ::MoveWindow(hwnd, r.left + dx, r.top + dy, (r.right - r.left) + dw, (r.bottom - r.top) + dh, bRepaint);
		}
	}

	return(bResult);
}

BOOL LeoHelpers::EnableDlgItem(HWND hwndDlg, int nIDDlgItem, BOOL bEnable)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		if (!bEnable && ::GetFocus() == hwnd)
		{
			// We're about to disable the control with focus. Give focus to the next control.
			// Set focus to next control. (Do not use SetFocus in dialogs. Use WM_NEXTDLGCTL instead.)
			::SendMessage(hwndDlg, WM_NEXTDLGCTL, 0, FALSE);
		}

		bResult = ::EnableWindow(hwnd, bEnable);
	}

	return(bResult);
}

BOOL LeoHelpers::ShowDlgItem(HWND hwndDlg, int nIDDlgItem, BOOL bShow)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		if (!bShow && ::GetFocus() == hwnd)
		{
			// We're about to hide the control with focus. Give focus to the next control.
			// Set focus to next control. (Do not use SetFocus in dialogs. Use WM_NEXTDLGCTL instead.)
			::SendMessage(hwndDlg, WM_NEXTDLGCTL, 0, FALSE);
		}

		bResult = ::SetWindowPos(hwnd, NULL, 0, 0, 0, 0,
			SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOSIZE|SWP_NOZORDER|(bShow?SWP_SHOWWINDOW:SWP_HIDEWINDOW));
	}

	return(bResult);
}

BOOL LeoHelpers::GetDlgItemText(HWND hwndDlg, int nIDDlgItem, std::wstring *pStrResult)
{
	BOOL bResult = FALSE;

	pStrResult->erase();

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		bResult = LeoHelpers::GetWindowTextString(hwnd, pStrResult, false);
	}

	return(bResult);
}

BOOL LeoHelpers::GetListBoxItemText(HWND hwndListBox, LRESULT lrItemIndex, std::wstring *pStrResult)
{
	BOOL bResult = FALSE;

	pStrResult->clear();

	LRESULT itemTextLength = ::SendMessage(hwndListBox, LB_GETTEXTLEN, lrItemIndex, 0);
	if (0 == itemTextLength)
	{
		bResult = TRUE;
	}
	else if (0 < itemTextLength)
	{
		wchar_t *szItemText = new wchar_t[ itemTextLength + 1 ];

		if (0 < ::SendMessage(hwndListBox, LB_GETTEXT, lrItemIndex, reinterpret_cast< LPARAM >(szItemText)))
		{
			*pStrResult = szItemText;
			bResult = TRUE;
		}

		delete [] szItemText;
	}

	return(bResult);
}

BOOL LeoHelpers::SetupComboItem(HWND hWndParent, int iDlgItem, bool bClear, DWORD dwValue, DWORD dwSelectValue, const wchar_t *szTitle)
{
	BOOL bResult = FALSE;

	HWND hWndCombo = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndCombo != NULL)
	{
		if (bClear)
		{
			::SendMessage(hWndCombo, CB_RESETCONTENT, 0, 0);
		}

		LRESULT idx = ::SendMessage(hWndCombo, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(szTitle));
		if (0 <= idx)
		{
		//	DWORD dwValue = *reinterpret_cast<DWORD *>(&iValue);

			::SendMessage(hWndCombo, CB_SETITEMDATA, idx, dwValue);

			if (dwSelectValue == dwValue)
			{
				::SendMessage(hWndCombo, CB_SETCURSEL, idx, 0);
			}

			bResult = TRUE;
		}
	}

	return(bResult);
}

BOOL LeoHelpers::SetComboItem(HWND hWndParent, int iDlgItem, DWORD dwValue)
{
	BOOL bResult = FALSE;

	HWND hWndCombo = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndCombo != NULL)
	{
		LRESULT itemCount = ::SendMessage(hWndCombo, CB_GETCOUNT, 0, 0);

		if (itemCount > 0)
		{
			DWORD dwItemValue = 0;

			for (LRESULT idx = 0; idx < itemCount; ++idx)
			{
				dwItemValue = static_cast< DWORD >( ::SendMessage(hWndCombo, CB_GETITEMDATA, idx, 0) );

				if (dwItemValue == dwValue)
				{
					::SendMessage(hWndCombo, CB_SETCURSEL, idx, 0);
					bResult = TRUE;
					break;
				}
			}
		}
	}

	return(bResult);
}

BOOL LeoHelpers::GetComboItem(HWND hWndParent, int iDlgItem, DWORD *pdwValue)
{
	BOOL bResult = FALSE;

	HWND hWndCombo = GetDlgItem(hWndParent, iDlgItem);

	if (hWndCombo != NULL)
	{
		LRESULT curSel = SendMessage(hWndCombo, CB_GETCURSEL, 0, 0);

		if (CB_ERR != curSel)
		{
			*pdwValue = static_cast< DWORD >(::SendMessage(hWndCombo, CB_GETITEMDATA, curSel, 0));

		//	iValue = *reinterpret_cast< int * >(&dwValue);

			bResult = TRUE;
		}
	}

	return(bResult);
}

BOOL LeoHelpers::GetComboItemIntCast(HWND hWndParent, int iDlgItem, int *piValue)
{
	DWORD dwValue = 0;
	if (!LeoHelpers::GetComboItem(hWndParent, iDlgItem, &dwValue))
	{
		return FALSE;
	}

	*piValue = *reinterpret_cast< int * >(&dwValue);
	return TRUE;
}

BOOL LeoHelpers::SetupCheckBoxItem(HWND hWndParent, int iDlgItem, bool bCheck)
{
	return(::CheckDlgButton(hWndParent, iDlgItem, bCheck ? BST_CHECKED : BST_UNCHECKED));
}

BOOL LeoHelpers::GetCheckBoxItem(HWND hWndParent, int iDlgItem, bool *pbValue)
{
	*pbValue = false;

//	UINT res = ::IsDlgButtonChecked(hWndParent, iDlgItem); <-- provides no proper failure indication.

	HWND hWndCheck = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndCheck == NULL)
	{
		return FALSE;
	}

	LRESULT res = ::SendMessage(hWndCheck, BM_GETCHECK, 0, 0);

	*pbValue = (BST_CHECKED == res);

	return TRUE;
}

#ifdef UDM_SETRANGE32
BOOL LeoHelpers::SetupSpinItem(HWND hWndParent, int iDlgItem, int iMax, int iMin, int iValue)
{
	BOOL bResult = FALSE;

	HWND hWndSpin = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndSpin != NULL)
	{
		::SendMessage(hWndSpin, UDM_SETRANGE32, iMin, iMax);
		::SendMessage(hWndSpin, UDM_SETPOS32, 0, iValue);
		// Recipient of WM_VSCROLL shouldn't rely on the 16-bit scroll amount.
		::SendMessage(hWndParent, WM_VSCROLL, MAKELONG(0, iValue), reinterpret_cast<LPARAM>(hWndSpin));

		bResult = TRUE;
	}

	return(bResult);
}

BOOL LeoHelpers::SetSpinItem(HWND hWndParent, int iDlgItem, int iValue)
{
	BOOL bResult = FALSE;

	HWND hWndSpin = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndSpin != NULL)
	{
		::SendMessage(hWndSpin, UDM_SETPOS32, 0, iValue);
		// Recipient of WM_VSCROLL shouldn't rely on the 16-bit scroll amount.
		::SendMessage(hWndParent, WM_VSCROLL, MAKELONG(0, iValue), reinterpret_cast<LPARAM>(hWndSpin));

		bResult = TRUE;
	}

	return(bResult);
}

BOOL LeoHelpers::GetSpinItem(HWND hWndParent, int iDlgItem, int *piValue)
{
	BOOL bResult = FALSE;

	HWND hWndSpin = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndSpin != NULL)
	{
		BOOL bError = TRUE;

		*piValue = static_cast< int >(::SendMessage(hWndSpin, UDM_GETPOS32, 0, reinterpret_cast<LPARAM>(&bError)));

		if (!bError)
		{
			bResult = TRUE;
		}
	}

	return(bResult);
}
#endif

BOOL LeoHelpers::SetupEditItem(HWND hWndParent, int iDlgItem, const wchar_t *szText)
{
	BOOL bResult = FALSE;

	HWND hWndEdit = ::GetDlgItem(hWndParent, iDlgItem);

	if (hWndEdit != NULL)
	{
		bResult = ::SetWindowText(hWndEdit, szText);
	}

	return(bResult);
}

BOOL LeoHelpers::GetEditItem(HWND hWndParent, int iDlgItem, std::wstring *pStrResult)
{
	return(LeoHelpers::GetDlgItemText(hWndParent, iDlgItem, pStrResult));
}

bool LeoHelpers::FilenameToEdit(HWND hwndDlg, int iDlgItem, const wchar_t *szTitle, const wchar_t *szFilter, bool bAddToRecent, DWORD *pdwCommDlgExtendedError)
{
	bool bChanged = false;

	if (pdwCommDlgExtendedError)
	{
		*pdwCommDlgExtendedError = 0;
	}

	wchar_t szBuffer[MAX_PATH];
	std::wstring strCurrent;
	LeoHelpers::GetDlgItemText(hwndDlg, iDlgItem, &strCurrent);
	LeoHelpers::StringCopy(szBuffer, strCurrent.c_str(), _countof(szBuffer));

	wchar_t * szFilterFixed = NULL;

	if (szFilter != NULL && szFilter[0] != L'\0')
	{
		// Opus language strings use ! where the filter nulls need to be, so fix up the string.
		// If there are no ! characters or there's an uneven number of them, use szFilter as-is since it
		// must already be in the right double-null-terminated format.

		const size_t filterLen = wcslen(szFilter);

		szFilterFixed = new wchar_t[ filterLen + 1 ];
		LeoHelpers::StringCopy(szFilterFixed, szFilter, filterLen + 1);

		size_t iFilterFixCount = 0;

		for (size_t i = 0; i < filterLen; ++i)
		{
			if (szFilterFixed[i] == L'!')
			{
				szFilterFixed[i] = L'\0';
				++iFilterFixCount;
			}
		}

		if (iFilterFixCount > 0 && (iFilterFixCount%2) == 0)
		{
			szFilter = szFilterFixed;
		}
	}

	OPENFILENAME ofn={0};
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwndDlg;
	ofn.hInstance = NULL;
	ofn.lpstrFilter = szFilter;
	ofn.lpstrFile = szBuffer;
	ofn.nMaxFile = _countof(szBuffer);
	ofn.lpstrTitle = szTitle;
	ofn.Flags = OFN_FILEMUSTEXIST|OFN_NOCHANGEDIR|OFN_LONGNAMES|OFN_HIDEREADONLY; // HIDEREADONLY hides the read-only checkbox/drop-down, not read-only files.

	if (!bAddToRecent)
	{
		ofn.Flags |= OFN_DONTADDTORECENT;
	}

	if (!::GetOpenFileName(&ofn))
	{
		if (pdwCommDlgExtendedError)
		{
			*pdwCommDlgExtendedError = ::CommDlgExtendedError();
		}
	}
	else
	{
		::SetDlgItemText(hwndDlg, iDlgItem, ofn.lpstrFile);
		bChanged = true;
	}

	delete[] szFilterFixed;

	return bChanged;
}

/*
BOOL LeoHelpers::GetDlgItemLong(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG *plResult)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		::SetLastError(0);

		LONG lStyleValue = ::GetWindowLong(hwnd, nIndex);

		if (0 != lStyleValue || 0 == ::GetLastError())
		{
			*plResult = lStyleValue;
			bResult = TRUE;
		}
	}

	return(bResult);
}

BOOL LeoHelpers::SetDlgItemLong(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG lValue)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		::SetLastError(0);

		LONG lOldStyleValue = ::SetWindowLong(hwnd, nIndex, lValue);

		if (0 != lOldStyleValue || 0 == ::GetLastError())
		{
			bResult = TRUE;
		}
	}

	return(bResult);
}
*/

BOOL LeoHelpers::GetDlgItemLongPtr(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG_PTR *plResult)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		::SetLastError(0);

		LONG_PTR lStyleValue = ::GetWindowLongPtr(hwnd, nIndex);

		if (0 != lStyleValue || 0 == ::GetLastError())
		{
			*plResult = lStyleValue;
			bResult = TRUE;
		}
	}

	return(bResult);
}

BOOL LeoHelpers::SetDlgItemLongPtr(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG_PTR lValue)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		::SetLastError(0);

		#pragma warning(suppress:4244) // spurious warning due to stupid Win32 SDK header definition.
		LONG_PTR lOldStyleValue = ::SetWindowLongPtr(hwnd, nIndex, lValue);

		if (0 != lOldStyleValue || 0 == ::GetLastError())
		{
			bResult = TRUE;
		}
	}

	return(bResult);
}

/*
BOOL LeoHelpers::ModifyDlgItemStyle(HWND hwndDlg, int nIDDlgItem, bool bStyleEx, LONG_PTR lAdd, LONG_PTR lRemove)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		int nStyleKey = bStyleEx ? GWL_EXSTYLE : GWL_STYLE;

		::SetLastError(0);

		LONG_PTR lStyleValue = ::GetWindowLongPtr(hwnd, nStyleKey);

		if (0 != lStyleValue || 0 == GetLastError())
		{
			lStyleValue &= (~lRemove);
			lStyleValue |= lAdd;

			lStyleValue = ::SetWindowLongPtr(hwnd, nStyleKey, lStyleValue);

			if (0 != lStyleValue || 0 == ::GetLastError())
			{
				bResult = TRUE;
			}
		}
	}

	return(bResult);
}
*/

LRESULT LeoHelpers::SendMessageDlgItem(HWND hwndDlg, int nIDDlgItem, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	LRESULT lResult = 0;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		lResult = ::SendMessage(hwnd, Msg, wParam, lParam);
	}

	return(lResult);
}

BOOL LeoHelpers::InvalidateDlgItem(HWND hwndDlg, int nIDDlgItem)
{
	BOOL bResult = FALSE;

	HWND hwnd = ::GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		bResult = ::InvalidateRect(hwnd, NULL, FALSE);
	}

	return(bResult);
}

BOOL LeoHelpers::EnableDlgControl(HWND hwndControl, BOOL bEnable)
{
	BOOL bResult = FALSE;

	if (NULL != hwndControl)
	{
		if (!bEnable && ::GetFocus() == hwndControl)
		{
			// We're about to disable the control with focus. Give focus to the next control.
			// Set focus to next control. (Do not use SetFocus in dialogs. Use WM_NEXTDLGCTL instead.)
			::SendMessage(GetParent(hwndControl), WM_NEXTDLGCTL, 0, FALSE);
		}

		bResult = ::EnableWindow(hwndControl, bEnable);
	}

	return(bResult);
}

HRESULT LeoHelpers::LoadIconMetricOrDefault(HINSTANCE hinst, PCWSTR pszName, int lims, HICON *phico, bool *pbDestroyIcon)
{
	HRESULT hr = E_FAIL;

	*pbDestroyIcon = false;

	HMODULE hModComCtl32 = ::LoadLibrary(L"comctl32.dll");

	if (hModComCtl32 != NULL)
	{
		typedef HRESULT (WINAPI *LOADICONMETRIC_PTR)(HINSTANCE hinst, PCWSTR pszName, int lims, __out HICON *phico);

		LOADICONMETRIC_PTR fLoadIconMetric = reinterpret_cast< LOADICONMETRIC_PTR >(::GetProcAddress(hModComCtl32, "LoadIconMetric"));

		if (fLoadIconMetric != NULL)
		{
			hr = fLoadIconMetric(hinst, pszName, lims, phico);
			*pbDestroyIcon = true; // Presumably the result of this is never shared, even if the icon wasn't resized, although MSDN doesn't mention anything about clean-up.
		}

		::FreeLibrary(hModComCtl32);
	}

	if (S_OK != hr)
	{
		int cx = 0;
		int cy = 0;

		if (LIM_SMALL == lims)
		{
			cx = ::GetSystemMetrics(SM_CXSMICON);
			cy = ::GetSystemMetrics(SM_CYSMICON);
		}
		else if (LIM_LARGE == lims)
		{
			cx = ::GetSystemMetrics(SM_CXICON);
			cy = ::GetSystemMetrics(SM_CYICON);
		}

		if (cx != 0 && cy != 0)
		{
			HICON hTemp = reinterpret_cast<HICON>(::LoadImage(hinst, pszName, IMAGE_ICON, cx, cy, 0));

			if (hTemp != NULL)
			{
				hr = S_OK;
				*phico = hTemp;
				*pbDestroyIcon = true;
			}
			else
			{
				hTemp = reinterpret_cast<HICON>(::LoadImage(hinst, pszName, IMAGE_ICON, cx, cy, LR_SHARED));

				if (hTemp != NULL)
				{
					hr = S_OK;
					*phico = hTemp;
					*pbDestroyIcon = false;
				}
			}
		}
	}

	if (S_OK != hr)
	{
		*phico = NULL;
		*pbDestroyIcon = false;
	}

	return hr;
}

LeoHelpers::ThemeHelper::ThemeHelper()
: m_hbmpDIB(NULL)
, m_hdcMemTemp(NULL)
, m_hBrushTemp(NULL)
{
}

LeoHelpers::ThemeHelper::~ThemeHelper()
{
	if (m_hBrushTemp != NULL)
	{
		DeleteObject(m_hBrushTemp);
		m_hBrushTemp = NULL;
	}

	if (m_hdcMemTemp != NULL)
	{
		DeleteDC(m_hdcMemTemp);
		m_hdcMemTemp = NULL;
	}

	if (NULL != m_hbmpDIB)
	{
		DeleteObject(m_hbmpDIB);
		m_hbmpDIB = NULL;
	}
}

bool LeoHelpers::ThemeHelper::EnableThemeDialogTexture(HWND hWnd, DWORD dwFlags)
{
	bool bResult = false;

	if (SUCCEEDED(::EnableThemeDialogTexture(hWnd, dwFlags)))
	{
		bResult = true;
	}

	return(bResult);
}

#ifdef LEOHELPERS_DEBUG_TABBACKGROUND
// Create a test pattern that makes it easy to spot when we're doing something wrong.
bool LeoHelpers::ThemeHelper::CacheBackgroundTabThemeDialog(HWND hWnd)
{
	bool bResult = false;

	if (NULL != m_hbmpDIB)
	{
		DeleteObject(m_hbmpDIB);
		m_hbmpDIB = NULL;
	}

//	if (IsAvailable())
	{
		HDC hdcMem = CreateCompatibleDC(NULL);

		if (NULL != hdcMem)
		{
			m_sizePart.cx = 128;
			m_sizePart.cy = 128;

			BITMAPINFO bmi;
			ZeroMemory(&bmi, sizeof(bmi));
			bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
			bmi.bmiHeader.biWidth = m_sizePart.cx;
			bmi.bmiHeader.biHeight = m_sizePart.cy * 2;
			bmi.bmiHeader.biPlanes = 1;
			bmi.bmiHeader.biBitCount = 24;
			bmi.bmiHeader.biCompression = BI_RGB;

			void *pDontCare = NULL;

			m_hbmpDIB = CreateDIBSection(hdcMem, &bmi, DIB_RGB_COLORS, &pDontCare, NULL, 0);

			if (NULL != m_hbmpDIB)
			{
				HGDIOBJ hbmpOld = SelectObject(hdcMem, m_hbmpDIB);

				if (NULL != hbmpOld)
				{
					static int iBrushType = HS_BDIAGONAL;

					HBRUSH hBrushNew = CreateHatchBrush(iBrushType, RGB(200,200,200));

					++iBrushType;
					iBrushType %= 6;

					if (hBrushNew != NULL)
					{
						RECT rcFill;
						::SetRect(&rcFill, 0, 0, bmi.bmiHeader.biWidth, bmi.bmiHeader.biHeight);

						if (FillRect(hdcMem, &rcFill, hBrushNew))
						{
							bResult = true;
						}

						DeleteObject(hBrushNew);
					}

					SelectObject(hdcMem, hbmpOld);
				}
			}

			DeleteDC(hdcMem);
		}
	}

	if (!bResult)
	{
		if (NULL != m_hbmpDIB)
		{
			DeleteObject(m_hbmpDIB);
			m_hbmpDIB = NULL;
		}
	}

	return bResult;
}
#else
bool LeoHelpers::ThemeHelper::CacheBackgroundTabThemeDialog(HWND hWnd)
{
	bool bResult = false;

	if (NULL != m_hbmpDIB)
	{
		DeleteObject(m_hbmpDIB);
		m_hbmpDIB = NULL;
	}

	HANDLE hTheme = ::OpenThemeData(hWnd, L"Tab");

	if (NULL != hTheme)
	{
		HDC hdcMem = CreateCompatibleDC(NULL);

		if (NULL != hdcMem)
		{
			if (SUCCEEDED(::GetThemePartSize(hTheme, hdcMem, TABP_BODY, 0, NULL, TS_TRUE, &m_sizePart)))
			{
				BITMAPINFO bmi;
				ZeroMemory(&bmi, sizeof(bmi));
				bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
				bmi.bmiHeader.biWidth = m_sizePart.cx;
				bmi.bmiHeader.biHeight = m_sizePart.cy * 2;
				bmi.bmiHeader.biPlanes = 1;
				bmi.bmiHeader.biBitCount = 24;
				bmi.bmiHeader.biCompression = BI_RGB;

				void *pDontCare = NULL;

				m_hbmpDIB = CreateDIBSection(hdcMem, &bmi, DIB_RGB_COLORS, &pDontCare, NULL, 0);

				if (NULL != m_hbmpDIB)
				{
					HGDIOBJ hbmpOld = SelectObject(hdcMem, m_hbmpDIB);

					if (NULL != hbmpOld)
					{
						// Get an upside-down copy of the bitmap so that we can draw over every second
						// line of right-way-up bitmaps, so that the stupid thing tiles.

						RECT rectBackgroud;
						rectBackgroud.left	 = 0;
						rectBackgroud.top	 = 0;
						rectBackgroud.right	 = m_sizePart.cx;
						rectBackgroud.bottom = m_sizePart.cy;

						if (SUCCEEDED(::DrawThemeBackground(hTheme, hdcMem, TABP_BODY, 0, &rectBackgroud, NULL)))
						{
							StretchBlt(hdcMem, 0, m_sizePart.cy * 2 - 1, m_sizePart.cx, -m_sizePart.cy,
							           hdcMem, 0, 0,                     m_sizePart.cx,  m_sizePart.cy, SRCCOPY);

							bResult = true;
						}

						SelectObject(hdcMem, hbmpOld);
					}
				}
			}

			DeleteDC(hdcMem);
		}

		::CloseThemeData(hTheme);
	}

	if (!bResult)
	{
		if (NULL != m_hbmpDIB)
		{
			DeleteObject(m_hbmpDIB);
			m_hbmpDIB = NULL;
		}
	}

	return bResult;
}
#endif

void LeoHelpers::ThemeHelper::EraseBackgroundTabThemeDialog(HWND hWnd, HDC hDC, const RECT &rcClient, const RECT &rcBackgroundWnd)
{
	if (NULL != m_hbmpDIB || CacheBackgroundTabThemeDialog(hWnd))
	{
		HBITMAP hbmpOld = NULL;

		HDC hdcMem = CreateCompatibleDC(hDC);

		if (NULL != hdcMem)
		{
			HBRUSH hBrushNew = CreatePatternBrush(m_hbmpDIB);

			if (NULL != hBrushNew)
			{
				POINT ptOldOrigin={0};

				// SetBrushOrgEx sets the origin for the next brush that gets selected.
				if (SetBrushOrgEx(hDC, rcBackgroundWnd.left, rcBackgroundWnd.top, &ptOldOrigin))
				{
					HGDIOBJ hBrushOld = SelectObject(hDC, hBrushNew);

					if (hBrushOld != NULL)
					{
						PatBlt(hDC, rcClient.left, rcClient.top, rcClient.right - rcClient.left, rcClient.bottom - rcClient.top, PATCOPY);
					}

					SetBrushOrgEx(hDC, ptOldOrigin.x, ptOldOrigin.y, NULL);

					if (hBrushOld != NULL)
					{
						SelectObject(hDC, hBrushOld);
					}
				}

				DeleteObject(hBrushNew);
			}

			DeleteDC(hdcMem);
		}
	}
	else
	{
		HBRUSH hBrushNotOurs = NULL;

		for (HWND hWndForBackground = hWnd; hWndForBackground != NULL; hWndForBackground = GetParent(hWndForBackground))
		{
			#pragma warning(suppress:4312) // spurious warning due to stupid Win32 SDK header definition.
			hBrushNotOurs = reinterpret_cast<HBRUSH>(::GetClassLongPtr(hWndForBackground, GCLP_HBRBACKGROUND));

			if (hBrushNotOurs != NULL
			||	!(GetWindowLongPtr(hWndForBackground, GWL_STYLE) & WS_CHILD))
			{
				break;
			}
		}

		if (hBrushNotOurs)
		{
			hBrushNotOurs = GetSysColorBrush(COLOR_3DFACE);
		}

		POINT ptOldOrigin={0};

		// SetBrushOrgEx sets the origin for the next brush that gets selected.
		if (hBrushNotOurs != NULL && SetBrushOrgEx(hDC, rcBackgroundWnd.left, rcBackgroundWnd.top, &ptOldOrigin))
		{
			HGDIOBJ hBrushOld = SelectObject(hDC, hBrushNotOurs);

			if (hBrushOld != NULL)
			{
				PatBlt(hDC, rcClient.left, rcClient.top, rcClient.right - rcClient.left, rcClient.bottom - rcClient.top, PATCOPY);
			}

			SetBrushOrgEx(hDC, ptOldOrigin.x, ptOldOrigin.y, NULL);

			if (hBrushOld != NULL)
			{
				SelectObject(hDC, hBrushOld);
			}
		}
	}
}

HBRUSH LeoHelpers::ThemeHelper::GetTempBrush(HWND hWnd, HDC hDC, const RECT &rcClient, const RECT &rcBackgroundWnd)
{
	if (m_hBrushTemp != NULL)
	{
		DeleteObject(m_hBrushTemp);
		m_hBrushTemp = NULL;
	}

	if (m_hdcMemTemp != NULL)
	{
		DeleteDC(m_hdcMemTemp);
		m_hdcMemTemp = NULL;
	}

	if (NULL != m_hbmpDIB || CacheBackgroundTabThemeDialog(hWnd))
	{
		HDC hdcMem = CreateCompatibleDC(hDC);

		if (hdcMem != NULL)
		{
			HBRUSH hBrushNew = CreatePatternBrush(m_hbmpDIB);

			if (NULL != hBrushNew)
			{
				// SetBrushOrgEx sets the origin for the next brush that gets selected.
				if (SetBrushOrgEx(hDC, rcBackgroundWnd.left, rcBackgroundWnd.top, NULL))
				{
					m_hBrushTemp = hBrushNew;
					m_hdcMemTemp = hdcMem;
					hBrushNew = NULL;
					hdcMem = NULL;
				}

				if (hBrushNew != NULL)
				{
					DeleteObject(hBrushNew);
				}
			}

			if (hdcMem != NULL)
			{
				DeleteDC(hdcMem);
			}
		}
	}
	else
	{
		#pragma warning(suppress:4312) // spurious warning due to stupid Win32 SDK header definition.
		HBRUSH hBrushNotOurs = reinterpret_cast<HBRUSH>(::GetClassLongPtr(hWnd, GCLP_HBRBACKGROUND));

		if (hBrushNotOurs == NULL)
		{
			hBrushNotOurs = GetSysColorBrush(COLOR_3DFACE);
		}

		// SetBrushOrgEx sets the origin for the next brush that gets selected.
		if (hBrushNotOurs != NULL && SetBrushOrgEx(hDC, rcBackgroundWnd.left, rcBackgroundWnd.top, NULL))
		{
			return hBrushNotOurs;
		}
	}

	return m_hBrushTemp;
}

/*
BOOL LeoHelpers::ThemeHelper::IsThemeBackgroundPartiallyTransparentButton(HWND hWndControl, int iPartId, int iStateId)
{
	if (!IsAvailable())
	{
		return FALSE;
	}

	const wchar_t szClassList = L"Button";

	if ((GetWindowLongPtr(hWndControl, GWL_STYLE) & BS_TYPEMASK) == BS_DEFPUSHBUTTON))
	{
		szClassList = L"OkButton;Button";
	}

	HTHEME hTheme = m_fOpenThemeData(hWndControl, szClassList);

	if (NULL == hTheme)
	{
		return FALSE;
	}

	BOOL bResult = m_fIsThemeBackgroundPartiallyTransparent(hTheme, iPartId, iStateId);

	m_fCloseThemeData(hTheme);

}
*/

HRESULT LeoHelpers::ThemeHelper::DrawThemeParentBackground(HWND hWnd, HDC hDC, const RECT *prcDrawChildCoords)
{
	return ::DrawThemeParentBackground(hWnd, hDC, prcDrawChildCoords);
}





LeoHelpers::CommonDialogData::CommonDialogData(void *pvUserData)
	: m_bShuttingDown(false)
	, m_pvUserData(pvUserData)
{
}

LeoHelpers::CommonDialogData::~CommonDialogData()
{
}

void LeoHelpers::CommonDialogData::AddTabBackgroundDlg(HWND hwndDlg)
{
	if (hwndDlg != NULL)
	{
		m_tabBackgroundDlgs.push_back(hwndDlg);
	}
}

// If you have any tabs your main dialog proc should call here as part of its WM_THEMECHANGED
// processing. You should also make one call during initialisation. Before any calls,
// use AddTabBackgroundDlg to specify the child dialogs that need tab-theme backgrounds.
void LeoHelpers::CommonDialogData::OnThemeChanged()
{
	if (!m_tabBackgroundDlgs.empty())
	{
		DWORD dwEnableFlags = ETDT_DISABLE;

		// Do this so we always regenerate the background image (or just clear it) when the theme changes.
		if (m_themeHelper.CacheBackgroundTabThemeDialog(m_tabBackgroundDlgs.front()))
		{
			dwEnableFlags = ETDT_ENABLETAB;
		}

		for (std::vector< HWND >::iterator wndIter = m_tabBackgroundDlgs.begin(); wndIter != m_tabBackgroundDlgs.end(); ++wndIter)
		{
			m_themeHelper.EnableThemeDialogTexture(*wndIter, dwEnableFlags);
		}
	}
}

// If you have a tab that contains a child dialog, the child dialog's WM_ERASEBKGND should call this
// passing its own window handle and the HDC passed to WM_ERASEBKGND's wParam argument, and return
// the result.
BOOL LeoHelpers::CommonDialogData::TabChildDlgEraseBackground(HWND hwndDlg, HDC hDC)
{
	RECT rcClient = {0};
	if (::GetClientRect(hwndDlg, &rcClient))
	{
		if (m_eraseBackgroundCallers.empty() || m_eraseBackgroundCallers.back()==NULL)
		{
			m_themeHelper.EraseBackgroundTabThemeDialog(hwndDlg, hDC, rcClient, rcClient);
			return TRUE;
		}
		else
		{
			HWND hwndControl = m_eraseBackgroundCallers.back();

			RECT rcControl = {0};

			if (::GetClientRect(hwndControl, &rcControl)
			&&	LeoHelpers::ClientToScreenRect(hwndDlg,     &rcClient)
			&&	LeoHelpers::ScreenToClientRect(hwndControl, &rcClient))
			{
				m_themeHelper.EraseBackgroundTabThemeDialog(hwndDlg, hDC, rcControl, rcClient);
				return TRUE;
			}
		}
	}
	return FALSE;
}

// If you have a tab that contains a child dialog, the child dialog's WM_CTLCOLORBTN and WM_CTLCOLORSTATIC
// should call this passing its own window handle and the HDC passed to the messages' wParam arguments,
// and return the result (cast to INT_PTR).
HBRUSH LeoHelpers::CommonDialogData::TabChildDlgCtlColor(HWND hwndDlg, HDC hDC)
{
	RECT rcClient = {0};
	if (::GetClientRect(hwndDlg, &rcClient))
	{
		if (m_eraseBackgroundCallers.empty() || m_eraseBackgroundCallers.back()==NULL)
		{
			HBRUSH hb = m_themeHelper.GetTempBrush(hwndDlg, hDC, rcClient, rcClient);

			::SetBkMode(hDC, TRANSPARENT);

			return hb;
		}
		else
		{
			HWND hwndControl = m_eraseBackgroundCallers.back();

			RECT rcControl = {0};

			if (::GetClientRect(hwndControl, &rcControl)
			&&	LeoHelpers::ClientToScreenRect(hwndDlg,     &rcClient)
			&&	LeoHelpers::ScreenToClientRect(hwndControl, &rcClient))
			{
				HBRUSH hb = m_themeHelper.GetTempBrush(hwndDlg, hDC, rcControl, rcClient);

				::SetBkMode(hDC, TRANSPARENT);

				return hb;
			}
		}
	}
	return reinterpret_cast< HBRUSH >( ::GetStockObject(NULL_BRUSH) );
}

// Subclassing with the doubleBufferPaintProc can remove flicker on resize and ensure tab-background textures are painted & aligned correctly.

void LeoHelpers::CommonDialogData::SubclassDoubleBufferPaintProc(HWND hwndCtl)
{
	::SetWindowSubclass(hwndCtl, staticDoubleBufferPaintProc, 0, reinterpret_cast< DWORD_PTR >( this ));
}

// Subclass with the backgroundAlignPaintProc can ensure tab-background textures are aligned correctly.

void LeoHelpers::CommonDialogData::SubclassBackgroundAlignPaintProc(HWND hwndCtl)
{
	::SetWindowSubclass(hwndCtl, staticBackgroundAlignPaintProc, 0, reinterpret_cast< DWORD_PTR >( this ));
}

// Subclassing with the noEraseListBoxPaintProc can remove flicker when resizing listbox controls, in their empty areas.

void LeoHelpers::CommonDialogData::SubclassNoEraseListBoxPaintProc(HWND hwndCtl)
{
	::SetWindowSubclass(hwndCtl, staticNoEraseListBoxPaintProc, 0, reinterpret_cast< DWORD_PTR >( this ));
}

LRESULT CALLBACK LeoHelpers::CommonDialogData::staticNoEraseListBoxPaintProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, UINT_PTR uIdSubclass, DWORD_PTR dwRefData)
{
	if (uMsg == WM_NCDESTROY)
	{
		::RemoveWindowSubclass(hWnd, staticNoEraseListBoxPaintProc, uIdSubclass);
		return ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
	}

	CommonDialogData *pThis = reinterpret_cast< CommonDialogData * >( dwRefData );

	return pThis->memberNoEraseListBoxPaintProc(hWnd, uMsg, wParam, lParam);
}

LRESULT LeoHelpers::CommonDialogData::memberNoEraseListBoxPaintProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_ERASEBKGND)
	{
		LRESULT itemCount = ::SendMessage(hWnd, LB_GETCOUNT, 0, 0);

		if (itemCount > 0) // If there are no items then let the control erase itself. Otherwise...
		{
			RECT rcClientErase = {0};
			if (::GetClientRect(hWnd, &rcClientErase))
			{
				RECT rcLastItem = {0};

				if (LB_ERR == ::SendMessage(hWnd, LB_GETITEMRECT, itemCount - 1, reinterpret_cast<LPARAM>(&rcLastItem))
				||	rcLastItem.bottom >= rcClientErase.bottom)
				{
					// Don't erase anything.
					return TRUE;
				}

				// Only erase below the bottom of the last item.
				if (rcLastItem.bottom > rcClientErase.top)
				{
					rcClientErase.top = rcLastItem.bottom;
				}

				HDC hDC = reinterpret_cast< HDC >(wParam);

				HBRUSH hBrushBackground = reinterpret_cast< HBRUSH >(
					::SendMessage(::GetParent(hWnd), WM_CTLCOLORLISTBOX,
						reinterpret_cast< WPARAM >(hDC), reinterpret_cast< LPARAM >(hWnd)));

				if (hBrushBackground != NULL)
				{
					::FillRect(hDC, &rcClientErase, hBrushBackground);
					return TRUE;
				}
			}
		}
	}

	return ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK LeoHelpers::CommonDialogData::staticBackgroundAlignPaintProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, UINT_PTR uIdSubclass, DWORD_PTR dwRefData)
{
	if (uMsg == WM_NCDESTROY)
	{
		::RemoveWindowSubclass(hWnd, staticBackgroundAlignPaintProc, uIdSubclass);
		return ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
	}

	CommonDialogData *pThis = reinterpret_cast< CommonDialogData * >( dwRefData );

	return pThis->memberBackgroundAlignPaintProc(hWnd, uMsg, wParam, lParam);
}

LRESULT LeoHelpers::CommonDialogData::memberBackgroundAlignPaintProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_PAINT)
	{
		m_eraseBackgroundCallers.push_back(hWnd);

		LRESULT lr = ::DefSubclassProc(hWnd, uMsg, wParam, lParam);

		m_eraseBackgroundCallers.pop_back();

		return lr;
	}

	return ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK LeoHelpers::CommonDialogData::staticDoubleBufferPaintProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, UINT_PTR uIdSubclass, DWORD_PTR dwRefData)
{
	if (uMsg == WM_NCDESTROY)
	{
		::RemoveWindowSubclass(hWnd, staticDoubleBufferPaintProc, uIdSubclass);
		return ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
	}

	CommonDialogData *pThis = reinterpret_cast< CommonDialogData * >( dwRefData );

	return pThis->memberDoubleBufferPaintProc(hWnd, uMsg, wParam, lParam);
}

LRESULT LeoHelpers::CommonDialogData::memberDoubleBufferPaintProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_PAINT)
	{
		RECT rcUpdate={0};

		m_eraseBackgroundCallers.push_back(hWnd);

		LRESULT lr = 0;

		if (::GetUpdateRect(hWnd, &rcUpdate, FALSE))
		{
			lr = onPaintDoubleBuffer(hWnd);
		}
		else
		{
			// Route internal paint messages (where there is no update region) to the original proc.
			lr = ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
		}

		m_eraseBackgroundCallers.pop_back();

		return lr;
	}
	else if (uMsg == WM_ERASEBKGND) // This is needed for the tab control background when using the Windows Classic theme.
	{
		return 0;
	}

	return ::DefSubclassProc(hWnd, uMsg, wParam, lParam);
}

LRESULT LeoHelpers::CommonDialogData::onPaintDoubleBuffer(HWND hWnd)
{
	PAINTSTRUCT paint={0};

	HDC hDC = ::BeginPaint(hWnd, &paint);

	if (hDC)
	{
		if (WS_VISIBLE & GetWindowLongPtr(hWnd,GWL_STYLE))
		{
			RECT rcClient={0};
			RECT rcParent={0};

			HWND hWndParent = ::GetParent(hWnd);

			if (hWndParent != NULL
			&&	::GetClientRect(hWnd, &rcClient)
			&&	::GetClientRect(hWndParent, &rcParent)
			&&	LeoHelpers::ClientToScreenRect(hWndParent, &rcParent)
			&&	LeoHelpers::ScreenToClientRect(hWnd, &rcParent))
			{
				if (WS_CLIPSIBLINGS & GetWindowLongPtr(hWnd,GWL_STYLE))
				{
					RECT rcSibling;

					for (HWND hwndSibling = GetWindow(hWnd, GW_HWNDFIRST); hwndSibling != NULL; hwndSibling = GetWindow(hwndSibling, GW_HWNDNEXT))
					{
						if (hwndSibling != hWnd)
						{
							if (::GetWindowRect(hwndSibling, &rcSibling)
							&&	LeoHelpers::ScreenToClientRect(hWnd, &rcSibling))
							{
								::ExcludeClipRect(hDC, rcSibling.left, rcSibling.top, rcSibling.right, rcSibling.bottom);
							}
						}
					}
				}

				if (!LeoHelpers::WantFullWindowDragging())
				{
					doubleBufferPaintContents(hDC, hWnd, hWndParent, rcParent, paint);
				}
				else
				{
					HDC hdcMem = ::CreateCompatibleDC(hDC);

					if (NULL != hdcMem)
					{
						HBITMAP hBitmapNew = ::CreateCompatibleBitmap(hDC, rcClient.right - rcClient.left, rcClient.bottom - rcClient.top);

						if (NULL != hBitmapNew)
						{
							HGDIOBJ hBitmapOld = ::SelectObject(hdcMem, hBitmapNew);

							if (NULL != hBitmapOld)
							{
								doubleBufferPaintContents(hdcMem, hWnd, hWndParent, rcParent, paint);

								::BitBlt(hDC, 0, 0, rcClient.right - rcClient.left, rcClient.bottom - rcClient.top,
										 hdcMem, 0, 0, SRCCOPY);

								::SelectObject(hdcMem, hBitmapOld);
							}

							::DeleteObject(hBitmapNew);
						}

						::DeleteDC(hdcMem);
					}

					GdiFlush();
				}
			}
		}

		::EndPaint(hWnd, &paint);
	}

	return 0;
}

void LeoHelpers::CommonDialogData::doubleBufferPaintContents(HDC hDC, HWND hWnd, HWND hWndParent, const RECT &rcParent, const PAINTSTRUCT &paint)
{
	wchar_t szClassName[32]; // Just needs to hold "SysTabControl32" (inc null).

	int gcnRes = GetClassName(hWnd, szClassName, _countof(szClassName));

	if (gcnRes > 0 && gcnRes < _countof(szClassName))
	{
		if (0 == LeoHelpers::WideStrCmpUpper(WC_BUTTON, szClassName)
		&&	((GetWindowLongPtr(hWnd, GWL_STYLE) & BS_TYPEMASK) == BS_GROUPBOX))
		{
			m_themeHelper.EraseBackgroundTabThemeDialog(hWnd, hDC, paint.rcPaint, rcParent);
		}
		else //if (0 == LeoHelpers::WideStrCmpUpper(WC_TABCONTROL, szClassName))
		{
			HBRUSH hBrushNotOurs = NULL;

			for (HWND hWndForBackground = hWnd; hWndForBackground != NULL; hWndForBackground = GetParent(hWndForBackground))
			{
				#pragma warning(suppress:4312) // spurious warning due to stupid Win32 SDK header definition.
				hBrushNotOurs = reinterpret_cast<HBRUSH>(::GetClassLongPtr(hWndForBackground, GCLP_HBRBACKGROUND));

				if (hBrushNotOurs != NULL
				||	!(GetWindowLongPtr(hWndForBackground, GWL_STYLE) & WS_CHILD))
				{
					break;
				}
			}

			if (hBrushNotOurs)
			{
				hBrushNotOurs = GetSysColorBrush(COLOR_3DFACE);
			}

			::FillRect(hDC, &paint.rcPaint, hBrushNotOurs);
		}
	}

	SendMessage(hWnd, WM_PRINT, reinterpret_cast<WPARAM>(hDC), /*PRF_CHILDREN |*/ PRF_CLIENT | PRF_NONCLIENT);
}

/*
void NDlgDocsConfig::childEraseBackground(ConfigData *pcd, HWND hwndChildDlg, HDC hDC, RECT *prectControl)
{
	RECT rcClient;
	if (!GetClientRect(hwndChildDlg, &rcClient))
	{
		return;
	}

	wchar_t szClassName[8]; // Just needs to hold "BUTTON" (inc null).

	for (HWND hwndCtrl = GetWindow(hwndChildDlg, GW_CHILD); hwndCtrl != NULL; hwndCtrl = GetWindow(hwndCtrl, GW_HWNDNEXT))
	{
		int gcnRes = GetClassName(hwndCtrl, szClassName, _countof(szClassName));

		if (gcnRes > 0 && gcnRes < _countof(szClassName))
		{
			if (0 == LeoHelpers::WideStrCmpUpper(WC_BUTTON, szClassName))
			{
			//	pcd->comDat.GetThemeHelper().DrawThemeParentBackground(hwndCtrl, hDC, NULL);

				// Do not exclude group boxes from the erasing as they don't erase their own backgrounds.
				continue;
			}
		}

		RECT rcCtrl;
		if (GetWindowRect(hwndCtrl, &rcCtrl))
		{
			LeoHelpers::ScreenToClientRect(hwndChildDlg, &rcCtrl);
			ExcludeClipRect(hDC, rcCtrl.left, rcCtrl.top, rcCtrl.right, rcCtrl.bottom);
		}
	}

	pcd->comDat.GetThemeHelper().EraseBackgroundTabThemeDialog(hwndChildDlg, hDC, prectControl ? *prectControl : rcClient, rcClient);
}
*/
