#pragma once

namespace LeoHelpers
{
	// The result may not be reliable for windows owned by other threads. (e.g. If they change between asking for the length and asking for the string.)
	// Set bUseTimeoutEvenIfSameProcess true if there is a chance of deadlock between your thread and the window's thread and they're in the same process.
	BOOL GetWindowTextString(HWND hwnd, std::wstring *pStrResult, bool bUseTimeoutEvenIfSameProcess);

	// Useful for debug messages more than anything else.
	// The class may be truncated for very long class names.
	// The title may not be reliable for windows owned by other threads. (e.g. If they change between asking for the length and asking for the string.)
	class WindowInfo
	{
	private:
		std::wstring m_strInfo;

		WindowInfo(const WindowInfo &rhs); // disallow
		WindowInfo &operator=(const WindowInfo &rhs); // disallow
	public:
		WindowInfo(HWND hwnd);
		virtual ~WindowInfo() { };
		const wchar_t *Get() const { return m_strInfo.c_str(); }
	};

	// Get top-level parent window
	HWND GetTopLevelWindow(HWND hWnd);

	// Give hwndThis=NULL to test the other window against the current thread.
	bool SameWindowThread(HWND hwndThis, HWND hwndOther);

	// Check if a window is hung. Can help avoid deadlocks when using multiple UI threads/processes.
	// Beware that this won't detect all deadlock situations: The window in question could have a child
	// or parent window running on another UI thread which may still deadlock with the calling thread
	// when something is done to hwnd which causes it to try and send a message to the other UI thread.
	// bBlock maps to SendMessageTimeout's SMTO_BLOCK and prevents the calling thread from processing
	// messages sent to it from other threads while waiting for the window in question to respond.
	// bAssert set true will fire an assert in debug builds if the window isn't responding.
	bool IsWindowResponding(HWND hwnd, bool bBlock, bool bAssert);
};
